"use strict";
/**
 * Copyright (c) 2023 mol* contributors, licensed under MIT, See LICENSE file for more info.
 *
 * @author Adam Midlik <midlik@gmail.com>
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.MVSJFormatProvider = exports.LoadMvsData = exports.ParseMVSJ = exports.Mvs = void 0;
const provider_1 = require("../../../mol-plugin-state/formats/provider");
const objects_1 = require("../../../mol-plugin-state/objects");
const data_1 = require("../../../mol-plugin-state/transforms/data");
const mol_state_1 = require("../../../mol-state");
const mol_task_1 = require("../../../mol-task");
const assets_1 = require("../../../mol-util/assets");
const param_definition_1 = require("../../../mol-util/param-definition");
const load_1 = require("../load");
const mvs_data_1 = require("../mvs-data");
const annotation_structure_component_1 = require("./annotation-structure-component");
/** Plugin state object storing `MVSData` */
class Mvs extends objects_1.PluginStateObject.Create({ name: 'MVS Data', typeClass: 'Data' }) {
}
exports.Mvs = Mvs;
/** Transformer for parsing data in MVSJ format */
exports.ParseMVSJ = (0, annotation_structure_component_1.MVSTransform)({
    name: 'mvs-parse-mvsj',
    display: { name: 'MVS Annotation Component', description: 'A molecular structure component defined by MVS annotation data.' },
    from: objects_1.PluginStateObject.Data.String,
    to: Mvs,
})({
    apply({ a }, plugin) {
        const mvsData = mvs_data_1.MVSData.fromMVSJ(a.data);
        const sourceUrl = tryGetDownloadUrl(a, plugin);
        return new Mvs({ mvsData, sourceUrl });
    },
});
/** If the PluginStateObject `pso` comes from a Download transform, try to get its `url` parameter. */
function tryGetDownloadUrl(pso, plugin) {
    var _a;
    const theCell = plugin.state.data.selectQ(q => q.ofTransformer(data_1.Download)).find(cell => cell.obj === pso);
    const urlParam = (_a = theCell === null || theCell === void 0 ? void 0 : theCell.transform.params) === null || _a === void 0 ? void 0 : _a.url;
    return urlParam ? assets_1.Asset.getUrl(urlParam) : undefined;
}
/** Params for the `LoadMvsData` action */
const LoadMvsDataParams = {
    replaceExisting: param_definition_1.ParamDefinition.Boolean(false, { description: 'If true, the loaded MVS view will replace the current state; if false, the MVS view will be added to the current state.' }),
};
/** State action which loads a MVS view into Mol* */
exports.LoadMvsData = mol_state_1.StateAction.build({
    display: { name: 'Load MVS Data' },
    from: Mvs,
    params: LoadMvsDataParams,
})(({ a, params }, plugin) => mol_task_1.Task.create('Load MVS Data', async () => {
    const { mvsData, sourceUrl } = a.data;
    await (0, load_1.loadMVS)(plugin, mvsData, { replaceExisting: params.replaceExisting, sourceUrl: sourceUrl });
}));
/** Data format provider for MVSJ format.
 * If Visuals:On, it will load the parsed MVS view;
 * otherwise it will just create a plugin state object with parsed data. */
exports.MVSJFormatProvider = (0, provider_1.DataFormatProvider)({
    label: 'MVSJ',
    description: 'MVSJ',
    category: 'Miscellaneous',
    stringExtensions: ['mvsj'],
    parse: async (plugin, data) => {
        return plugin.state.data.build().to(data).apply(exports.ParseMVSJ).commit();
    },
    visuals: async (plugin, data) => {
        const ref = mol_state_1.StateObjectRef.resolveRef(data);
        const params = param_definition_1.ParamDefinition.getDefaultValues(LoadMvsDataParams);
        return await plugin.state.data.applyAction(exports.LoadMvsData, params, ref).run();
    },
});
